/*
 * Methods for the class Explorer, for class definition see Explorer.h
 * This file is part of assignment 4 of Application Development.
 * http://vanslooten.com/appdev
*/

#include "Explorer.h"

void Explorer::init(EVShield * s, NewPing * p, EVs_NXTTouch * t) {
  /**
   * Initialize various variables.
   */
    evshield = s;
    sonar = p;
    touch = t;
  
    evshield->init( SH_HardwareI2C );
    touch->init( evshield, SH_BBS2);
    // reset motors
    evshield->bank_a.motorReset();
    evshield->bank_b.motorReset();
    evshield->ledSetRGB(165, 255, 0); // led orange (indicates ready for driving)
}


void Explorer::drive(int distance = 0) { // distance = 0 means drive unlimited
  /*
   * Drive the robot. This method has one parameter: distance, which has a default value of 0. Which means that
   * if no parameter is given, the value of distance will be 0.
   * The method can be called like this:
   * drive()      - to drive unlimited
   * drive(30)    - drive forward for 30cm (then stop)
   * drive(-10)   - drive reverse for 10cm (then stop)
   */
  if (distance == 0) {
    Serial.println("drive unlimited");
    evshield->bank_a.motorRunUnlimited( SH_Motor_Both, motor_direction, speed );
    isDriving = true;
    evshield->ledSetRGB(0, 255, 0); // led green (indicates driving)
  }
  else {
    Serial.print("drive distance:"); Serial.println(distance);
    double circumference = WHEEL_DIAM * PI;
    unsigned int degrees = (abs(distance)/circumference) * 360;
    Serial.print("degrees="); Serial.println(degrees);
    if (distance>0) { // positive distance, so just go in the current direction
      evshield->bank_a.motorRunDegrees(SH_Motor_Both, motor_direction, speed, degrees, SH_Completion_Wait_For, SH_Next_Action_Float ); // SH_Next_Action_BrakeHold
    }
    else {
      // negative distance, so go other direction:
      SH_Direction use_motor_direction = SH_Direction_Forward;
      if (motor_direction==SH_Direction_Forward) use_motor_direction=SH_Direction_Reverse;
      evshield->bank_a.motorRunDegrees(SH_Motor_Both, use_motor_direction, speed, degrees, SH_Completion_Wait_For, SH_Next_Action_Float );
    }
    isDriving = false;
    evshield->ledSetRGB(165, 255, 0); // led orange (indicates ready for driving)
  }
}

void Explorer::stop() {
  isDriving = false;
  evshield->bank_a.motorStop(SH_Motor_Both, SH_Next_Action_Brake); // stop using SH_Next_Action=SH_Next_Action_Float (alt: SH_Next_Action_Brake SH_Next_Action_BrakeHold)
  evshield->ledSetRGB(165, 255, 0); // led orange (indicates ready for driving)
  delay(1000);
}

void Explorer::turn(int angle) {
/*
 * Rotate the Explorer around its own axis (point turn) with the given angle (in degrees).
 * So to turn the Explorer 90 degrees use: turn(90)
 * To turn into the opplosite direction use: turn(-90)
 */
  // calculate turn angle:
  unsigned int degrees = abs(angle) * (TRACKWIDTH / WHEEL_DIAM);
  Serial.print("Turn degrees: "); Serial.println(degrees);
  if (angle>0) {
    evshield->bank_a.motorRunDegrees(SH_Motor_1, SH_Direction_Forward, speed, degrees, SH_Completion_Dont_Wait, SH_Next_Action_Float);
    evshield->bank_a.motorRunDegrees(SH_Motor_2, SH_Direction_Reverse, speed, degrees, SH_Completion_Wait_For, SH_Next_Action_Float);
  }
  else {
    evshield->bank_a.motorRunDegrees(SH_Motor_2, SH_Direction_Forward, speed, degrees, SH_Completion_Dont_Wait, SH_Next_Action_Float);
    evshield->bank_a.motorRunDegrees(SH_Motor_1, SH_Direction_Reverse, speed, degrees, SH_Completion_Wait_For, SH_Next_Action_Float);
  }
}

void Explorer::reverseDirection() {
  /**
   * Stop and reverse the direction we are driving (go back).
   */
  stop();
  forward = !forward;
  if (forward) {
    motor_direction = SH_Direction_Forward;
    Serial.println("drive forward");
  }
  else {
    motor_direction = SH_Direction_Reverse;
    Serial.println("drive reverse");
  }
  drive();
}


void Explorer::checkSensors() {
  if ( evshield->getButtonState(BTN_LEFT) ) { // change driving direction
     reverseDirection();
  }
  else if ( evshield->getButtonState(BTN_RIGHT) ) { // drive 1m
     Serial.println("drive 1m");
     stop();
     drive(100);
  }
  else if ( evshield->getButtonState(BTN_GO) ) { // start/stop
    isDriving = !isDriving;
    if (isDriving) {
      drive();
    }
    else {
      stop();
    }
  }
}

