/*
 * Explorer.h
 * This is a base class for the Explorer robot.
 * In this file, the class Explorer is defined, and it's methods are declared.
 * In C++ methods are declared seperatly from their implementation in a header file (.h).
 * 
 * This file is part of assignment 4 of Application Development.
 * http://vanslooten.com/appdev
*/

#ifndef Explorer_H
#define Explorer_H

#define WHEEL_DIAM 4.96 // wheel diameter in cm
#define TRACKWIDTH 14   // track width in cm

#define TRIGGER_PIN  3  // Arduino pin tied to trigger pin on the ultrasonic sensor.
#define ECHO_PIN     5  // Arduino pin tied to echo pin on the ultrasonic sensor.
#define MAX_DISTANCE 300 // Maximum distance we want to ping for (in centimeters). Maximum sensor distance is rated at 400-500cm.


#include <Wire.h>
#include <EVShield.h>
#include <EVs_NXTTouch.h>

#include <NewPing.h>

/**
  @brief This class interfaces with EVShield to create an explorer robot.
  Setup:
    - Two motors to drive, connected to Bank A of the EVShield
    - One motor on which the ultrasonic sensor is mounted: M1 on Bank B
    - Touch sensor on port BBS2 (Bank B)
*/
class Explorer {
  // class variables:
private:
  // pointers to objects created in main sketch:
  EVShield * evshield;
  NewPing * sonar;
  EVs_NXTTouch * touch;

public:
  bool isDriving = false;   // false = not driving, true = driving, we start in 'not driving' state
  bool forward = true;  // we start driving forward
  SH_Direction motor_direction = SH_Direction_Reverse; // initial motor direction (depends on how motors are mounted)
  int speed = SH_Speed_Slow; // SH_Speed_Medium
  SH_Motor sensorMotor = SH_Motor_1; // first motor (M1) on Bank B on which the ultrasonic sensor is mounted

  // methods:

  void init(EVShield * s, NewPing * p, EVs_NXTTouch * t);

  // driving related methods:
  void reverseDirection();
  void drive(int distance = 0);
  void stop();
  void turn(int angle);

  // sensor related methods:
  void checkSensors();
};

#endif
